/* Copyright (C) 2007-2010 STS Soft

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; version 2 of the License.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

using System.Collections;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;

namespace STSdb.Data
{
    public partial class FTable<TKey, TRecord>
    {
        internal abstract class BaseCollection : IEnumerable<Row<byte[], TRecord>>
        {
            public readonly Link Owner;
            public readonly uint Level;

            public BaseCollection(Link owner)
            {
                Debug.Assert(owner != null);

                Owner = owner;
                Level = Owner.Parent == null ? 0 : Owner.Parent.Collection.Level + 1;
            }

            public abstract TRecord this[byte[] index] { get; set; }

            public abstract bool TryGet(byte[] index, out TRecord record);
            public abstract void Delete(byte[] index);
            public abstract void Modify(byte[] index);

            public abstract IEnumerable<Row<byte[], TRecord>> Forward(byte[] firstIndex, byte[] lastIndex);
            public abstract IEnumerable<Row<byte[], TRecord>> Backward(byte[] lastIndex, byte[] firstIndex);

            #region IEnumerable<Row<byte[],TRecord>> Members

            public abstract IEnumerator<Row<byte[], TRecord>> GetEnumerator();

            #endregion

            #region IEnumerable Members

            IEnumerator IEnumerable.GetEnumerator()
            {
                return GetEnumerator();
            }

            #endregion

            public abstract void Save(MemoryStream stream);
            public abstract void Load(MemoryStream stream);
        }
    }
}
