﻿/* Copyright (C) 2007-2010 STS Soft

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; version 2 of the License.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

using System;
using System.IO;
using STSdb.General.Collections;

namespace STSdb.Persist
{
    public class DateTimeIndexerPersist : IIndexerPersist<DateTime>
    {
        public readonly Int64DeltaIndexerPersist persist = new Int64DeltaIndexerPersist();
        
        #region IIndexerPersist<T> Members

        public void Store(BinaryWriter writer, IIndexer<DateTime> values)
        {
            persist.Store(writer, new InternalIndexer(values));
        }

        public void Load(BinaryReader reader, IIndexer<DateTime> values)
        {
            persist.Load(reader, new InternalIndexer(values));
        }

        #endregion

        private class InternalIndexer : IIndexer<long>
        {
            private IIndexer<DateTime> indexer;

            public InternalIndexer(IIndexer<DateTime> indexer)
            {
                this.indexer = indexer;
            }

            #region IIndexer<long> Members

            public long this[int index]
            {
                get { return indexer[index].Ticks; }
                set { indexer[index] = new DateTime(value); }
            }

            public int Count
            {
                get { return indexer.Count; }
            }

            #endregion
        }
    }
}
