﻿/* Copyright (C) 2007-2010 STS Soft

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; version 2 of the License.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using STSdb.General.Collections;

namespace STSdb.Persist
{
    public class ImageIndexerPersist : IIndexerPersist<Image>
    {
        public readonly MemoryStreamIndexerPersist persist = new MemoryStreamIndexerPersist();
        
        #region IIndexerPersist<Image> Members

        public void Store(BinaryWriter writer, IIndexer<Image> values)
        {
            persist.Store(writer, new InternalIndexer(values));
        }

        public void Load(BinaryReader reader, IIndexer<Image> values)
        {
            persist.Load(reader, new InternalIndexer(values));
        }

        #endregion

        private class InternalIndexer : IIndexer<MemoryStream>
        {
            private IIndexer<Image> indexer;

            public InternalIndexer(IIndexer<Image> indexer)
            {
                this.indexer = indexer;
            }

            #region IIndexer<MemoryStream> Members

            public MemoryStream this[int index]
            {
                get
                {
                    Image image = indexer[index];
                    if (image == null)
                        return null;

                    MemoryStream stream = new MemoryStream();
                    //image.Save(stream, image.RawFormat);//TODO: Another bullshit of Microsoft
                    image.Save(stream, ImageFormat.Png);
                    return stream;
                }
                set
                {
                    if (value != null && value.Length > 0)
                        indexer[index] = Image.FromStream(value);
                    else
                        indexer[index] = null;
                }
            }

            public int Count
            {
                get { return indexer.Count; }
            }

            #endregion
        }
    }
}
