﻿/* Copyright (C) 2007-2010 STS Soft

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; version 2 of the License.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

using System;
using System.IO;
using STSdb.General.Collections;

namespace STSdb.Persist
{
    public class MemoryStreamIndexerPersist : IIndexerPersist<MemoryStream>
    {
        public readonly ByteArrayIndexerPersist persist = new ByteArrayIndexerPersist();
        
        #region IIndexerPersist<MemoryStream> Members

        public void Store(BinaryWriter writer, IIndexer<MemoryStream> values)
        {
            persist.Store(writer, new InternalIndexer(values));
        }

        public void Load(BinaryReader reader, IIndexer<MemoryStream> values)
        {
            persist.Load(reader, new InternalIndexer(values));
        }

        #endregion

        private class InternalIndexer : IIndexer<Byte[]>
        {
            private IIndexer<MemoryStream> indexer;

            public InternalIndexer(IIndexer<MemoryStream> indexer)
            {
                this.indexer = indexer;
            }

            #region IIndexer<Byte[]> Members

            public Byte[] this[int index]
            {
                get
                {
                    MemoryStream stream = indexer[index];
                    if (stream == null)
                        return null;

                    return stream.ToArray();
                }
                set
                {
                    if (value != null)
                        indexer[index] = new MemoryStream(value);
                    else
                        indexer[index] = null;
                }
            }

            public int Count
            {
                get { return indexer.Count; }
            }

            #endregion
        }
    }
}
